

# tree.string.from.nexus() below reads a nexus 
# file containing a concensus MUL-tree (with annotations
# and numbers for tip names) and converts to a plain Newick string 
# with tip names in the string and no annotations. 




library(ape)
library(stringr)





glue <- function(...) {
  paste(...,sep="")
}


# extracts a tree from a nexus file as a text string
# and a vector of tip names. The text string is 
# "tree TREE1 = [&R]" or similar followed by Newick string
tree.from.nexus <- function(filepath) {
x <- scan(filepath, what="", sep="\n", quiet=TRUE)
begin.trees.index <- grep("BEGIN TREES;", x=x, ignore.case=TRUE)
all.trees <- x[(begin.trees.index +1):length(x)]
end.trees.index <-  grep("END;|ENDBLOCK;", x=all.trees, ignore.case=TRUE)
all.trees <- all.trees[1:(end.trees.index-1)]
first.tree.index <- grep("TREE", x=all.trees, ignore.case=TRUE)
first.tree <- all.trees[first.tree.index]
translate.block <- all.trees[1:(first.tree.index-1)] 
#print(translate.block)
ntips <- length(translate.block) - 2
stopifnot(grepl("TRANSLATE", translate.block[1], ignore.case=TRUE))
stopifnot(grepl(";", translate.block[ntips+2], ignore.case=TRUE))
translate.block <- translate.block[2:(ntips+1)]
tipnameslist  <-  strsplit(translate.block, "[ \t,]")
#print(tipnameslist)
tipnames <- rep("", ntips)
for (i in 1:ntips) {
  tipnames[i] <- tipnameslist[[i]][length(tipnameslist[[i]])]
  }
list(tree.string=first.tree, tipnames=tipnames)
}


# removes annotations from a tree. The input is a text string
# containing (ending with) a representation of the tree in Newick 
# format with annotations as Newick comments (text in square brackets).
# Something like
# "tree TREE1 = [&R] ((2[&height_95%_HPD={0.0,1.734723475976807E-18},   ....   ,length=0.0];" 
# becomes
# "((2:0.00799792515647312,(4:0.005862847207511892,(8:0.0030000316544969733,6:0.0030000316544969733):
#   0.002862815553014919):0.002135077948961228):0.003737076067775142,((3:0.005862847207511892,(7:
#   0.0030000316544969733,5:0.0030000316544969733):0.002862815553014919):0.002964172734050802,1:
#   0.008827019941562694):0.002907981282685568);"
remove.annotations <- function(string) {
noannotbitslist <- strsplit(string, "[[][^][]*[]]")
#print(noannotbitslist)
# [[] matches [
# [^][]* matches anything without ] or [
# []] matches ]
noannotbits <- paste(noannotbitslist[[1]], sep="", collapse="")
#cat("noannotbits\n")
#print(noannotbits)
str_extract(noannotbits, "[(][^;]*;")
}   


# replaces tip node numbers 1,2,... with tip names
# from tipnames[]. Eg "14" becomes tipnames[14] in string.
# It works by looking for colons : and finding digit(s) just before.
# It will fail if > 99 tips.
putin.tipnames <- function(string, tipnames) {
  stopifnot(length(tipnames) < 100)
  chvect <- unlist(strsplit(string, ""))
  for (i in 1:length(chvect)) {
    if (chvect[i] == ":") {
      n <- 0 
      if (i>1  &&  grepl("[0123456789]", chvect[i-1])) {
        n <- as.integer(chvect[i-1])
        if (i>2  &&  grepl("[0123456789]", chvect[i-2])) {
          n <- 10*as.integer(chvect[i-2]) + n
          chvect[i-2] <- ""
        }
      }
      if (n > 0) {
        stopifnot(n <= length(tipnames))
        chvect[i-1] <- tipnames[n]
      }
    }
  }
  paste(chvect, sep="", collapse="")
}



tree.string.from.nexus <- function(fpath) {
newicktreewithanns <- tree.from.nexus(fpath)
tree.string.withnums <- remove.annotations(newicktreewithanns$tree.string)
tree.string <- putin.tipnames(tree.string.withnums, newicktreewithanns$tipnames)
tree.string
}



